class CallerPackageException(Exception):
    def __init__(self, message="Failed to get the caller package directory"):
        self.message = message
        super().__init__(self.message)


class MissingHeaderException(Exception):
    def __init__(self, header_name, message="Missing required header"):
        self.header_name = header_name
        self.message = f"{message}: {header_name}"
        super().__init__(self.message)


# -*- coding: utf-8 -*-

service_list = {}


class Response(object):
    # TYPE_LIST = {"JSON": "application/json", "ROW": "text/xml", "FORM": "application/x-www-form-urlencoded"}
    __body__ = ''
    __headers__ = []
    __status__ = 200
    __status_msg__ = ''
    __is_static__ = False

    # __type__ = 'JSON'

    def set_status(self, status, msg):
        self.__status__ = status
        self.__status_msg__ = msg

    def get_status(self):
        return self.__status__

    def set_body(self, data):
        self.__body__ = data

    def add_header(self, name, value):
        self.__headers__.append([name, value])

    def set_is_static(self, boolean):
        self.__is_static__ = boolean

    def get_is_static(self):
        return self.__is_static__

    # def set_type(self, response_type):
    #     if response_type == self.TYPE_LIST["JSON"]:
    #         self.__type__ = response_type
    #     if response_type == self.TYPE_LIST["ROW"]:
    #         self.__type__ = response_type
    #     if response_type == self.TYPE_LIST["FORM"]:
    #         self.__type__ = response_type

    def get_body(self):
        return self.__body__

    def get_headers(self):
        return self.__headers__

    # def get_type(self):
    #     return self.__type__


class Service(object):
    response = Response()

    # @abstractmethod
    # def service(self):
    #     pass
    #
    # @abstractmethod
    # def set_info(self):
    #     pass


class StatusResponse(Response):
    def __init__(self, **kwargs):
        if kwargs.get("status", None) is not None and kwargs.get("status_msg", None) is not None:
            self.set_status(kwargs["status"], kwargs["status_msg"])
        if kwargs.get("headers", None) is not None:
            for key, value in kwargs["headers"]:
                self.add_header(key, value)
        if kwargs.get("body", None) is not None:
            self.set_body(kwargs["body"])


def response_build(**kwargs):
    return StatusResponse(**kwargs)


def service(cls):
    if service_list.get(cls.__name__, None) is None:
        service_list[cls.__name__] = cls()



from typing import Callable
from cullinan.controller import url_resolver, EncapsulationHandler


def websocket(**kwargs) -> Callable:
    url = kwargs.get('url', '')
    global url_params
    url_params = None
    if url != '':
        url, url_params = url_resolver(url)

    def inner(cls):
        EncapsulationHandler.add_url_ws(url, cls)

    return inner



from cullinan.exceptions import MissingHeaderException


class MissingHeaderHandlerHook:

    @staticmethod
    def miss_header_handler(request, header_name):
        raise MissingHeaderException(request, header_name)

    hook = miss_header_handler

    @classmethod
    def set_hook(cls, hook):
        cls.hook = hook

    @classmethod
    def get_hook(cls):
        return cls.hook



# -*- coding: utf-8 -*-

from setuptools import setup

with open("README.MD", "r", encoding="UTF-8") as fh:
    long_description = fh.read()

setup(
    name='cullinan',
    version='0.56',
    packages=['cullinan'],
    description='Cullinan is written based on tornado and Sqlalchemy to help the project quickly build web application',
    author='plumeink',
    project_urls={
        'Source': 'https://github.com/plumeink/Cullinan',
        'Wiki': 'https://github.com/plumeink/Cullinan/wiki',
    },
    long_description=long_description,
    long_description_content_type="text/markdown",
    author_email='official@plumeink.com',
    url='https://github.com/plumeink/Cullinan',
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    license='http://www.apache.org/licenses/LICENSE-2.0',
    install_requires=['tornado', 'python-dotenv', 'sqlalchemy', 'pymysql'],
    python_requires='>=3'
)
